% Anne Collins
% Brown University, Providence, RI, USA
% Laboratory for Neural Computation and Cognition
% Michael Frank.

% November 2013:
% OpAL model simulation
% Submitted to Psych Review


% this function is called by OpAL_Code_Demo to simulate PST and effort
% tasks.

% you can also call it on its own, for example:
% OpAL([.1 .1 .1],[1 1 1 1 .5 .5],.8,1,100,.5,1)
% will plot model values over time for given parameters

% inputs:
% 1: para = [alphaG, alphaN, alphaC] (learning rates)
% 2: Init = [G(B), G(A), N(B), N(A), Q(B), Q(A)]. Initial model values.
% 3: seuil = p(r=val|A) = 1-p(r=val|B). probability of reward.
% 4: val: value of reward (usually 1)
% 5: duration: .5*number of trials
% 6: (optional) sample: probability of choosing A
% 7: (optional) 0: do not plot. 1: plot evolution of model values over
% time.

% outputs:
%output = [G(B), G(A), N(B), N(A), Q(B), Q(A)]. Final model values.
        

function output = OpAL(para,Init,seuil,val,duration,sample, flag_figure)

if nargin <7
    % don't plot unless explicitly told to
    flag_figure = 0;
end

% get model parameters
alphaG = para(1);
alphaN = para(2);
alphaC = para(3);
betaG = 10*para(1);
betaN = 10*para(2);

% iterations over which to average
nRep = 1000;

% loop over iterations
for rep =1:nRep
    tosave = [];
    % number of trials to run: 2* duration
    trials = 2*duration;
    % initialize model values
    QG = Init(1:2);% G weights
    QN = Init(3:4);% N weights
    Q = Init(5:6);% critic values
    for t = 1:trials
        tosave(t,:) = [Q QG QN];
        % if not forced to choose in a given way from input, sample by
        % softmax
        if nargin<6
        sample =1./(1+exp(1*(betaG*QG(1)-betaN*QN(1)-betaG*QG(2)+betaN*QN(2))));
        end
        % choose from policy an option
        c = (rand<sample);% 1: correct (2nd choice), 0: incorrect (1st choice)
        
        % sample for reward
        if rand < seuil; r = val; else r = 0; end
        
        % get reward depending on correct or not
        r = c*r +(1-c)*(val-r);
        % prediction error (chosen is correct+1)
        delta = r - Q(c+1);
        
        % update critic
        Q(c+1) = Q(c+1) + alphaC*delta;
        % update G, N values
        QG(c+1) = QG(c+1) + alphaG*(+delta)*(QG(c+1));%
        QN(c+1) = QN(c+1) + alphaN*(-delta)*(QN(c+1));%
    end
    % store model values
    file(rep,:,:) = tosave;

end
pl = squeeze(mean(file));
co = {'k','g','r'};
nom = {'Q','G','N','Act'};

if flag_figure ==1

figure
subplot(6,1,1)
hold on
plot(pl(:,2),'color',[.5 .5 .5],'linewidth',2)
plot(pl(:,1),'color',[.5 .5 .5],'linewidth',1)
title(nom{1})
hold off
subplot(6,1,2)
hold on
plot(pl(:,4),'g','linewidth',2)
plot(pl(:,3),'g','linewidth',1)
title(nom{2})
hold off
subplot(6,1,3)
hold on
plot(pl(:,6),'r','linewidth',2)
plot(pl(:,5),'r','linewidth',1)
title(nom{3})
hold off
subplot(6,1,4)
hold on
plot(pl(:,4)-pl(:,6),'-k','linewidth',2)
plot(pl(:,3)-pl(:,5),'-k','linewidth',1)
plot([1:10:length(pl(:,4))],1*pl(1:10:end,4)-1*pl(1:10:end,6),'.k')
plot([1:10:length(pl(:,4))],1*pl(1:10:end,3)-1*pl(1:10:end,5),'.k')
hold off
title([nom{4},' \beta_G = \beta_N'])
subplot(6,1,5)
hold on
plot(1.5*pl(:,4)-.5*pl(:,6),'k','linewidth',2)
plot(1.5*pl(:,3)-.5*pl(:,5),'k','linewidth',1)
plot([1:10:length(pl(:,4))],1.5*pl(1:10:end,4)-.5*pl(1:10:end,6),'.g')
plot([1:10:length(pl(:,4))],1.5*pl(1:10:end,3)-.5*pl(1:10:end,5),'.g')
hold off
title([nom{4},' \beta_G > \beta_N'])
subplot(6,1,6)
hold on
plot(.5*pl(:,4)-1.5*pl(:,6),'k','linewidth',2)
plot(.5*pl(:,3)-1.5*pl(:,5),'k','linewidth',1)
plot([1:10:length(pl(:,4))],.5*pl(1:10:end,4)-1.5*pl(1:10:end,6),'.r')
plot([1:10:length(pl(:,4))],.5*pl(1:10:end,3)-1.5*pl(1:10:end,5),'.r')
hold off
title([nom{4},' \beta_G < \beta_N'])
hold off

end

output = pl(end,:);

end