% Anne Collins
% Brown University, Providence, RI, USA
% Laboratory for Neural Computation and Cognition
% Michael Frank.

% November 2013:
% RL Gain Loss model simulation
% Submitted to Psych Review


% this function is called by OpAL_Code_Demo to simulate PST task.

% you can also call it on its own, for example:
% RLGainLoss([.1 .1],[1 1 1 1 .5 .5],.8,1,100,.5,1)
% will plot model values over time for given parameters

% inputs:
% 1: para = [alphaG, alphaN, alphaC] (learning rates)
% 2: Init = [.,.,.,., Q(B), Q(A)]. Initial model values.
% 3: seuil = p(r=val|A) = 1-p(r=val|B). probability of reward.
% 4: val: value of reward (usually 1)
% 5: duration: .5*number of trials
% 6: (optional) sample: probability of choosing A
% 7: (optional) 0: do not plot. 1: plot evolution of model values over
% time.

% outputs:
%output = [.,.,.,., Q(B), Q(A)]. Final model values.


function output = RLGainLoss(para,Init,seuil,val,duration,sample, flag_figure)

if nargin <7
    % don't plot unless explicitly told to
    flag_figure = 0;
end

% gain and loss learning rates
alphaG = para(1);
alphaL = para(2);

% iterations over which to average
nRep = 1000;

% loop over iterations
for rep =1:nRep
    tosave = [];
    trials = 2*duration;
    Q = Init(5:6);
    for t = 1:trials
        tosave(t,:) = [Q];
        % if not forced to choose in a given way from input, sample by
        % softmax
        if nargin<6
        sample =1./(1+exp(Q(1)-Q(2)));
        end
        % choose from policy an option
        c = (rand<sample);% 1: correct (2nd choice), 0: incorrect (1st choice)
        
        % sample for reward
        if rand < seuil; r = val; else r = 0; end
        
        % get reward depending on correct or not
        r = c*r +(1-c)*(val-r);

        % prediction error (chosen is correct+1)
        delta = r - Q(c+1);
        % update with gain and loss learning rate
        Q(c+1) = Q(c+1) + alphaG*max(delta,0) + alphaL*min(delta,0);
        end
        

    file(rep,:,:) = tosave;

end
pl = squeeze(mean(file));
co = {'k','g','r'};
nom = {'Q'};

% plot model value
if flag_figure ==1

figure
hold on
plot(pl(:,2),'color',[.5 .5 .5],'linewidth',2)
plot(pl(:,1),'color',[.5 .5 .5],'linewidth',1)
title(nom{1})
hold off
ylabel('Q')
legend('B','A')
end

output = pl(end,:);

end